@@ -156,13 +156,14 @@ local function get_lua_version(flags)
    if flags["lua-version"] then
       return flags["lua-version"] 
    end
-   local dirs = {
-      dir.path(cfg.home_tree, ".luarocks"),
-      cfg.sysconfdir,
-   }
+   local dirs = {}
    if flags["project-tree"] then
-      table.insert(dirs, 1, dir.path(flags["project-tree"], "..", ".luarocks"))
+      table.insert(dirs, dir.path(flags["project-tree"], "..", ".luarocks"))
    end
+   if cfg.home_tree then
+      table.insert(dirs, dir.path(cfg.home_tree, ".luarocks"))
+   end
+   table.insert(dirs, cfg.sysconfdir)
    for _, d in ipairs(dirs) do
       local f = dir.path(d, "default-lua-version.lua")
       local mod, err = loadfile(f, "t")
